@@ -8,6 +8,6 @@ end
 rootpath = System.get_env("RELEASE_ROOT", rootdefault)
 config :ejabberd,
   file: Path.join(rootpath, "conf/ejabberd.yml"),
-  log_path: Path.join(rootpath, 'logs/ejabberd.log')
+  log_path: Path.join(rootpath, "logs/ejabberd.log")
 config :mnesia,
-  dir: Path.join(rootpath, 'database/')
+  dir: Path.join(rootpath, "database/")
