@@ -5,7 +5,7 @@
 %%% Created :  7 May 2006 by Mickael Remond <mremond@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -126,15 +126,19 @@ get_commands_spec() ->
 			module = ?MODULE, function = restart,
 			args = [], result = {res, rescode}},
      #ejabberd_commands{name = reopen_log, tags = [logs],
-			desc = "Reopen the log files after being renamed",
-			longdesc = "This can be useful when an external tool is "
+			desc = "Reopen maybe the log files after being renamed",
+			longdesc = "Has no effect on ejabberd main log files, "
+			"only on log files generated by some modules.\n"
+			"This can be useful when an external tool is "
 			"used for log rotation. See "
-			"https://docs.ejabberd.im/admin/guide/troubleshooting/#log-files",
+			"[Log Files](https://docs.ejabberd.im/admin/guide/troubleshooting/#log-files).",
 			policy = admin,
 			module = ?MODULE, function = reopen_log,
 			args = [], result = {res, rescode}},
      #ejabberd_commands{name = rotate_log, tags = [logs],
-			desc = "Rotate the log files",
+			desc = "Rotate maybe log file of some module",
+			longdesc = "Has no effect on ejabberd main log files, "
+			"only on log files generated by some modules.",
 			module = ?MODULE, function = rotate_log,
 			args = [], result = {res, rescode}},
      #ejabberd_commands{name = stop_kindly, tags = [server],
@@ -157,9 +161,10 @@ get_commands_spec() ->
                         result = {levelatom, atom}},
      #ejabberd_commands{name = set_loglevel, tags = [logs],
 			desc = "Set the loglevel",
+			longdesc = "Possible loglevels: `none`, `emergency`, `alert`, `critical`,
+			           `error`, `warning`, `notice`, `info`, `debug`.",
 			module = ?MODULE, function = set_loglevel,
-			args_desc = ["Desired logging level: none | emergency | alert | critical "
-				     "| error | warning | notice | info | debug"],
+			args_desc = ["Desired logging level"],
 			args_example = ["debug"],
 			args = [{loglevel, string}],
 			result = {res, rescode}},
@@ -171,7 +176,8 @@ get_commands_spec() ->
 			result_example = ["mod_configure", "mod_vcard"],
 			result = {modules, {list, {module, string}}}},
      #ejabberd_commands{name = update, tags = [server],
-			desc = "Update the given module, or use the keyword: all",
+			desc = "Update the given module",
+			longdesc = "To update all the possible modules, use `all`.",
 			module = ?MODULE, function = update,
 			args_example = ["mod_vcard"],
 			args = [{module, string}],
@@ -373,7 +379,7 @@ get_commands_spec() ->
 			result = {res, rescode}},
      #ejabberd_commands{name = set_master, tags = [cluster],
 			desc = "Set master node of the clustered Mnesia tables",
-			longdesc = "If you provide as nodename `self`, this "
+			longdesc = "If `nodename` is set to `self`, then this "
 			"node will be set as its own master.",
 			module = ?MODULE, function = set_master,
 			args_desc = ["Name of the erlang node that will be considered master of this node"],
