@@ -2,7 +2,7 @@
 %%% Created :  8 Dec 2016 by Evgeny Khramtsov <ekhramtsov@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -40,7 +40,10 @@
 	 handle_stream_start/2, handle_stream_end/2,
 	 handle_unauthenticated_packet/2, handle_authenticated_packet/2,
 	 handle_auth_success/4, handle_auth_failure/4, handle_send/3,
-	 handle_recv/3, handle_cdata/2, handle_unbinded_packet/2]).
+	 handle_recv/3, handle_cdata/2, handle_unbinded_packet/2,
+	 inline_stream_features/1, handle_sasl2_inline/2,
+	 handle_sasl2_inline_post/3, handle_bind2_inline/2,
+	 handle_bind2_inline_post/3, sasl_options/1]).
 %% Hooks
 -export([handle_unexpected_cast/2, handle_unexpected_call/3,
 	 process_auth_result/3, reject_unauthenticated_packet/2,
@@ -212,7 +215,12 @@ open_session(#{user := U, server := S, resource := R,
 	       Pres -> get_priority_from_presence(Pres)
 	   end,
     Info = [{ip, IP}, {conn, Conn}, {auth_module, AuthModule}],
-    ejabberd_sm:open_session(SID, U, S, R, Prio, Info),
+    case State of
+	#{bind2_session_id := Tag} ->
+	    ejabberd_sm:open_session(SID, U, S, R, Prio, Info, Tag);
+	_ ->
+	    ejabberd_sm:open_session(SID, U, S, R, Prio, Info)
+    end,
     xmpp_stream_in:establish(State2).
 
 %%%===================================================================
@@ -381,6 +389,9 @@ unauthenticated_stream_features(#{lserver := LServer}) ->
 authenticated_stream_features(#{lserver := LServer}) ->
     ejabberd_hooks:run_fold(c2s_post_auth_features, LServer, [], [LServer]).
 
+inline_stream_features(#{lserver := LServer}) ->
+    ejabberd_hooks:run_fold(c2s_inline_features, LServer, {[], []}, [LServer]).
+
 sasl_mechanisms(Mechs, #{lserver := LServer, stream_encrypted := Encrypted} = State) ->
     Type = ejabberd_auth:store_type(LServer),
     Mechs1 = ejabberd_option:disable_sasl_mechanisms(LServer),
@@ -407,6 +418,12 @@ sasl_mechanisms(Mechs, #{lserver := LServer, stream_encrypted := Encrypted} = St
 	 (_) -> false
       end, Mechs -- Mechs1).
 
+sasl_options(#{lserver := LServer}) ->
+    case ejabberd_option:disable_sasl_scram_downgrade_protection(LServer) of
+	true -> [{scram_downgrade_protection, false}];
+	_ -> []
+    end.
+
 get_password_fun(_Mech, #{lserver := LServer}) ->
     fun(U) ->
 	    ejabberd_auth:get_password_with_authmodule(U, LServer)
@@ -533,6 +550,22 @@ handle_cdata(Data, #{lserver := LServer} = State) ->
     ejabberd_hooks:run_fold(c2s_handle_cdata, LServer,
 			    State, [Data]).
 
+handle_sasl2_inline(Els, #{lserver := LServer} = State) ->
+    ejabberd_hooks:run_fold(c2s_handle_sasl2_inline, LServer,
+			    {State, Els, []}, []).
+
+handle_sasl2_inline_post(Els, Results, #{lserver := LServer} = State) ->
+    ejabberd_hooks:run_fold(c2s_handle_sasl2_inline_post, LServer,
+			    State, [Els, Results]).
+
+handle_bind2_inline(Els, #{lserver := LServer} = State) ->
+    ejabberd_hooks:run_fold(c2s_handle_bind2_inline, LServer,
+			    {State, Els, []}, []).
+
+handle_bind2_inline_post(Els, Results, #{lserver := LServer} = State) ->
+    ejabberd_hooks:run_fold(c2s_handle_bind2_inline_post, LServer,
+			    State, [Els, Results]).
+
 handle_recv(El, Pkt, #{lserver := LServer} = State) ->
     ejabberd_hooks:run_fold(c2s_handle_recv, LServer, State, [El, Pkt]).
 
