@@ -1,5 +1,5 @@
 %%%----------------------------------------------------------------------
-%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -140,6 +140,8 @@ opt_type(default_ram_db) ->
     econf:enum([mnesia, sql, redis]);
 opt_type(define_macro) ->
     econf:any();
+opt_type(disable_sasl_scram_downgrade_protection) ->
+    econf:bool();
 opt_type(disable_sasl_mechanisms) ->
     econf:list_or_single(
       econf:and_then(
@@ -563,6 +565,7 @@ options() ->
      {cluster_backend, mnesia},
      {cluster_nodes, []},
      {define_macro, []},
+     {disable_sasl_scram_downgrade_protection, false},
      {disable_sasl_mechanisms, []},
      {domain_balancing, #{}},
      {ext_api_headers, <<>>},
@@ -606,7 +609,7 @@ options() ->
      {log_modules_fully, []},
      {max_fsm_queue, undefined},
      {modules, []},
-     {negotiation_timeout, timer:seconds(30)},
+     {negotiation_timeout, timer:seconds(120)},
      {net_ticktime, timer:seconds(60)},
      {new_sql_schema, ?USE_NEW_SQL_SCHEMA_DEFAULT},
      {update_sql_schema, false},
