@@ -5,7 +5,7 @@
 %%% Created :  8 Dec 2004 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -56,7 +56,8 @@
 	 init_mssql/1,
 	 keep_alive/2,
 	 to_list/2,
-	 to_array/2]).
+	 to_array/2,
+         parse_mysql_version/2]).
 
 %% gen_fsm callbacks
 -export([init/1, handle_event/3, handle_sync_event/4,
@@ -66,12 +67,22 @@
 -export([connecting/2, connecting/3,
 	 session_established/2, session_established/3]).
 
+-ifdef(OTP_RELEASE).
+    -if(?OTP_RELEASE >= 27).
+	-type(odbc_connection_reference() ::  odbc:connection_reference()).
+    -else.
+	-type(odbc_connection_reference() ::  pid()).
+    -endif.
+-else.
+    -type(odbc_connection_reference() ::  pid()).
+-endif.
+
 -include("logger.hrl").
 -include("ejabberd_sql_pt.hrl").
 -include("ejabberd_stacktrace.hrl").
 
 -record(state,
-	{db_ref               :: undefined | pid(),
+	{db_ref               :: undefined | pid() | odbc_connection_reference(),
 	 db_type = odbc       :: pgsql | mysql | sqlite | odbc | mssql,
 	 db_version           :: undefined | non_neg_integer() | {non_neg_integer(), atom(), non_neg_integer()},
 	 reconnect_count = 0  :: non_neg_integer(),
@@ -1121,19 +1132,48 @@ mysql_to_odbc(ok) ->
 mysql_item_to_odbc(Columns, Recs) ->
     {selected, [element(2, Column) || Column <- Columns], Recs}.
 
-to_odbc({selected, Columns, Recs}) ->
-    Rows = [lists:map(
-	      fun(I) when is_integer(I) ->
-		      integer_to_binary(I);
-		 (B) ->
-		      B
-	      end, Row) || Row <- Recs],
-    {selected, [list_to_binary(C) || C <- Columns], Rows};
+to_odbc({selected, Columns, Rows}) ->
+    Rows2 = lists:map(
+	fun(Row) ->
+	    Row2 = if is_tuple(Row) -> tuple_to_list(Row);
+		       is_list(Row) -> Row
+		   end,
+	    lists:map(
+		fun(I) when is_integer(I) -> integer_to_binary(I);
+		    (B) -> B
+		end, Row2)
+	end, Rows),
+    {selected, [list_to_binary(C) || C <- Columns], Rows2};
 to_odbc({error, Reason}) when is_list(Reason) ->
     {error, list_to_binary(Reason)};
 to_odbc(Res) ->
     Res.
 
+parse_mysql_version(SVersion, DefaultUpsert) ->
+    case re:run(SVersion, <<"(\\d+)\\.(\\d+)(?:\\.(\\d+))?(?:-([^-]*))?">>,
+                [{capture, all_but_first, binary}]) of
+        {match, [V1, V2, V3, Type]} ->
+            V = ((bin_to_int(V1)*1000)+bin_to_int(V2))*1000+bin_to_int(V3),
+            TypeA = binary_to_atom(Type, utf8),
+            Flags = case TypeA of
+                        'MariaDB' -> DefaultUpsert;
+                        _ when V >= 5007026 andalso V < 8000000 -> 1;
+                        _ when V >= 8000020 -> 1;
+                        _ -> DefaultUpsert
+                    end,
+            {ok, {V, TypeA, Flags}};
+        {match, [V1, V2, V3]} ->
+            V = ((bin_to_int(V1)*1000)+bin_to_int(V2))*1000+bin_to_int(V3),
+            Flags = case V of
+                        _ when V >= 5007026 andalso V < 8000000 -> 1;
+                        _ when V >= 8000020 -> 1;
+                        _ -> DefaultUpsert
+                    end,
+            {ok, {V, unknown, Flags}};
+        _ ->
+            error
+    end.
+
 get_db_version(#state{db_type = pgsql} = State) ->
     case pgsql:squery(State#state.db_ref,
                       <<"select current_setting('server_version_num')">>) of
@@ -1159,27 +1199,10 @@ get_db_version(#state{db_type = mysql, host = Host} = State) ->
 					    [{timeout, 5000},
 					     {result_type, binary}])) of
 	{selected, _, [SVersion]} ->
-	    case re:run(SVersion, <<"(\\d+)\\.(\\d+)(?:\\.(\\d+))?(?:-([^-]*))?">>,
-			[{capture, all_but_first, binary}]) of
-		{match, [V1, V2, V3, Type]} ->
-		    V = ((bin_to_int(V1)*1000)+bin_to_int(V2))*1000+bin_to_int(V3),
-		    TypeA = binary_to_atom(Type, utf8),
-		    Flags = case TypeA of
-				'MariaDB' -> DefaultUpsert;
-				_ when V >= 5007026 andalso V < 8000000 -> 1;
-				_ when V >= 8000020 -> 1;
-				_ -> DefaultUpsert
-			    end,
-		    State#state{db_version = {V, TypeA, Flags}};
-		{match, [V1, V2, V3]} ->
-		    V = ((bin_to_int(V1)*1000)+bin_to_int(V2))*1000+bin_to_int(V3),
-		    Flags = case V of
-				_ when V >= 5007026 andalso V < 8000000 -> 1;
-				_ when V >= 8000020 -> 1;
-				_ -> DefaultUpsert
-			    end,
-		    State#state{db_version = {V, unknown, Flags}};
-		_ ->
+            case parse_mysql_version(SVersion, DefaultUpsert) of
+                {ok, V} ->
+                    State#state{db_version = V};
+                error ->
 		    ?WARNING_MSG("Error parsing mysql version: ~p", [SVersion]),
 		    State
 	    end;
