@@ -5,7 +5,7 @@
 %%% Created : 15 Aug 2023 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -28,10 +28,12 @@
 -author('alexey@process-one.net').
 
 -export([start/1, update_schema/3,
-         get_table_schema/2, get_table_indices/2, test/0]).
+         get_table_schema/2, get_table_indices/2, print_schema/3,
+         test/0]).
 
 -include("logger.hrl").
 -include("ejabberd_sql_pt.hrl").
+-include("ejabberd_ctl.hrl").
 
 start(Host) ->
     case should_update_schema(Host) of
@@ -40,8 +42,17 @@ start(Host) ->
                 true ->
                     ok;
                 false ->
-                    Table = filter_table_sh(schema_table()),
-                    Res = create_table(Host, Table),
+                    SchemaInfo =
+                        ejabberd_sql:sql_query(
+                          Host,
+                          fun(DBType, DBVersion) ->
+                                  #sql_schema_info{
+                                     db_type = DBType,
+                                     db_version = DBVersion,
+                                     new_schema = ejabberd_sql:use_new_schema()}
+                          end),
+                    Table = filter_table_sh(SchemaInfo, schema_table()),
+                    Res = create_table(Host, SchemaInfo, Table),
                     case Res of
                         {error, Error} ->
                             ?ERROR_MSG("Failed to create table ~s: ~p",
@@ -249,8 +260,8 @@ table_exists(Host, Table) ->
               end
       end).
 
-filter_table_sh(Table) ->
-    case {ejabberd_sql:use_new_schema(), Table#sql_table.name} of
+filter_table_sh(SchemaInfo, Table) ->
+    case {SchemaInfo#sql_schema_info.new_schema, Table#sql_table.name} of
         {true, _} ->
             Table;
         {_, <<"route">>} ->
@@ -351,12 +362,11 @@ guess_version(Host, Schemas) ->
                   fun(Schema) ->
                           lists:all(
                             fun(Table) ->
-                                    Table2 = filter_table_sh(Table),
                                     CurrentColumns =
                                         get_table_schema(
-                                          Host, Table2#sql_table.name),
+                                          Host, Table#sql_table.name),
                                     check_columns_compatibility(
-                                      Table2#sql_table.columns,
+                                      Table#sql_table.columns,
                                       CurrentColumns)
                             end, Schema#sql_schema.tables)
                   end, Schemas),
@@ -384,7 +394,7 @@ get_current_version(Host, Module, Schemas) ->
             Version
     end.
 
-format_type(pgsql, _DBVersion, Column) ->
+format_type(#sql_schema_info{db_type = pgsql}, Column) ->
     case Column#sql_column.type of
         text -> <<"text">>;
         {text, _} -> <<"text">>;
@@ -398,7 +408,7 @@ format_type(pgsql, _DBVersion, Column) ->
         {char, N} -> [<<"character(">>, integer_to_binary(N), <<")">>];
         bigserial -> <<"bigserial">>
     end;
-format_type(sqlite, _DBVersion, Column) ->
+format_type(#sql_schema_info{db_type = sqlite}, Column) ->
     case Column#sql_column.type of
         text -> <<"text">>;
         {text, _} -> <<"text">>;
@@ -412,7 +422,7 @@ format_type(sqlite, _DBVersion, Column) ->
         {char, N} -> [<<"character(">>, integer_to_binary(N), <<")">>];
         bigserial -> <<"integer primary key autoincrement">>
     end;
-format_type(mysql, _DBVersion, Column) ->
+format_type(#sql_schema_info{db_type = mysql}, Column) ->
     case Column#sql_column.type of
         text -> <<"text">>;
         {text, big} -> <<"mediumtext">>;
@@ -430,7 +440,7 @@ format_type(mysql, _DBVersion, Column) ->
         bigserial -> <<"bigint auto_increment primary key">>
     end.
 
-format_default(pgsql, _DBVersion, Column) ->
+format_default(#sql_schema_info{db_type = pgsql}, Column) ->
     case Column#sql_column.type of
         text -> <<"''">>;
         {text, _} -> <<"''">>;
@@ -444,7 +454,7 @@ format_default(pgsql, _DBVersion, Column) ->
         %{char, N} -> <<"''">>;
         %bigserial -> <<"0">>
     end;
-format_default(sqlite, _DBVersion, Column) ->
+format_default(#sql_schema_info{db_type = sqlite}, Column) ->
     case Column#sql_column.type of
         text -> <<"''">>;
         {text, _} -> <<"''">>;
@@ -458,7 +468,7 @@ format_default(sqlite, _DBVersion, Column) ->
         %{char, N} -> <<"''">>;
         %bigserial -> <<"0">>
     end;
-format_default(mysql, _DBVersion, Column) ->
+format_default(#sql_schema_info{db_type = mysql}, Column) ->
     case Column#sql_column.type of
         text -> <<"('')">>;
         {text, _} -> <<"('')">>;
@@ -473,20 +483,20 @@ format_default(mysql, _DBVersion, Column) ->
         %bigserial -> <<"0">>
     end.
 
-escape_name(pgsql, _DBVersion, <<"type">>) ->
+escape_name(#sql_schema_info{db_type = pgsql}, <<"type">>) ->
     <<"\"type\"">>;
-escape_name(_DBType, _DBVersion, ColumnName) ->
+escape_name(_SchemaInfo, ColumnName) ->
     ColumnName.
 
-format_column_def(DBType, DBVersion, Column) ->
+format_column_def(SchemaInfo, Column) ->
     [<<"    ">>,
-     escape_name(DBType, DBVersion, Column#sql_column.name), <<" ">>,
-     format_type(DBType, DBVersion, Column),
+     escape_name(SchemaInfo, Column#sql_column.name), <<" ">>,
+     format_type(SchemaInfo, Column),
      <<" NOT NULL">>,
      case Column#sql_column.default of
          false -> [];
          true ->
-             [<<" DEFAULT ">>, format_default(DBType, DBVersion, Column)]
+             [<<" DEFAULT ">>, format_default(SchemaInfo, Column)]
      end,
      case lists:keyfind(sql_references, 1, Column#sql_column.opts) of
          false -> [];
@@ -512,7 +522,7 @@ format_mysql_index_column(Table, ColumnName) ->
             ColumnName
     end.
 
-format_create_index(pgsql, _DBVersion, Table, Index) ->
+format_create_index(#sql_schema_info{db_type = pgsql}, Table, Index) ->
     TableName = Table#sql_table.name,
     Unique =
         case Index#sql_index.unique of
@@ -529,7 +539,7 @@ format_create_index(pgsql, _DBVersion, Table, Index) ->
        <<", ">>,
        Index#sql_index.columns),
      <<");">>];
-format_create_index(sqlite, _DBVersion, Table, Index) ->
+format_create_index(#sql_schema_info{db_type = sqlite}, Table, Index) ->
     TableName = Table#sql_table.name,
     Unique =
         case Index#sql_index.unique of
@@ -546,7 +556,7 @@ format_create_index(sqlite, _DBVersion, Table, Index) ->
        <<", ">>,
        Index#sql_index.columns),
      <<");">>];
-format_create_index(mysql, _DBVersion, Table, Index) ->
+format_create_index(#sql_schema_info{db_type = mysql}, Table, Index) ->
     TableName = Table#sql_table.name,
     Unique =
         case Index#sql_index.unique of
@@ -568,67 +578,129 @@ format_create_index(mysql, _DBVersion, Table, Index) ->
          end, Index#sql_index.columns)),
      <<");">>].
 
-format_create_table(pgsql = DBType, DBVersion, Table) ->
+format_primary_key(#sql_schema_info{db_type = mysql}, Table) ->
+    case lists:filter(
+           fun(#sql_index{meta = #{primary_key := true}}) -> true;
+              (_) -> false
+           end, Table#sql_table.indices) of
+        [] -> [];
+        [I] ->
+            [[<<"    ">>,
+              <<"PRIMARY KEY (">>,
+              lists:join(
+                <<", ">>,
+                lists:map(
+                  fun(Col) ->
+                          format_mysql_index_column(Table, Col)
+                  end, I#sql_index.columns)),
+              <<")">>]]
+    end;
+format_primary_key(_SchemaInfo, Table) ->
+    case lists:filter(
+           fun(#sql_index{meta = #{primary_key := true}}) -> true;
+              (_) -> false
+           end, Table#sql_table.indices) of
+        [] -> [];
+        [I] ->
+            [[<<"    ">>,
+              <<"PRIMARY KEY (">>,
+              lists:join(<<", ">>, I#sql_index.columns),
+              <<")">>]]
+    end.
+
+format_add_primary_key(#sql_schema_info{db_type = sqlite} = SchemaInfo,
+                       Table, Index) ->
+    format_create_index(SchemaInfo, Table, Index);
+format_add_primary_key(#sql_schema_info{db_type = pgsql}, Table, Index) ->
+    TableName = Table#sql_table.name,
+    [<<"ALTER TABLE ">>, TableName, <<" ADD PRIMARY KEY (">>,
+     lists:join(
+       <<", ">>,
+       Index#sql_index.columns),
+     <<");">>];
+format_add_primary_key(#sql_schema_info{db_type = mysql}, Table, Index) ->
+    TableName = Table#sql_table.name,
+    [<<"ALTER TABLE ">>, TableName, <<" ADD PRIMARY KEY (">>,
+     lists:join(
+       <<", ">>,
+       lists:map(
+         fun(Col) ->
+                 format_mysql_index_column(Table, Col)
+         end, Index#sql_index.columns)),
+     <<");">>].
+
+format_create_table(#sql_schema_info{db_type = pgsql} = SchemaInfo, Table) ->
     TableName = Table#sql_table.name,
     [iolist_to_binary(
        [<<"CREATE TABLE ">>, TableName, <<" (\n">>,
         lists:join(
           <<",\n">>,
           lists:map(
-            fun(C) -> format_column_def(DBType, DBVersion, C) end,
-            Table#sql_table.columns)),
+            fun(C) -> format_column_def(SchemaInfo, C) end,
+            Table#sql_table.columns) ++
+              format_primary_key(SchemaInfo, Table)),
         <<"\n);\n">>])] ++
-        lists:map(
-          fun(I) ->
-                  iolist_to_binary(
-                    [format_create_index(DBType, DBVersion, Table, I),
-                     <<"\n">>])
+        lists:flatmap(
+          fun(#sql_index{meta = #{primary_key := true}}) ->
+                  [];
+             (#sql_index{meta = #{ignore := true}}) ->
+                  [];
+             (I) ->
+                  [iolist_to_binary(
+                     [format_create_index(SchemaInfo, Table, I),
+                      <<"\n">>])]
           end,
           Table#sql_table.indices);
-format_create_table(sqlite = DBType, DBVersion, Table) ->
+format_create_table(#sql_schema_info{db_type = sqlite} = SchemaInfo, Table) ->
     TableName = Table#sql_table.name,
     [iolist_to_binary(
        [<<"CREATE TABLE ">>, TableName, <<" (\n">>,
         lists:join(
           <<",\n">>,
           lists:map(
-            fun(C) -> format_column_def(DBType, DBVersion, C) end,
-            Table#sql_table.columns)),
+            fun(C) -> format_column_def(SchemaInfo, C) end,
+            Table#sql_table.columns) ++
+              format_primary_key(SchemaInfo, Table)),
         <<"\n);\n">>])] ++
-        lists:map(
-          fun(I) ->
-                  iolist_to_binary(
-                    [format_create_index(DBType, DBVersion, Table, I),
-                     <<"\n">>])
+        lists:flatmap(
+          fun(#sql_index{meta = #{primary_key := true}}) ->
+                  [];
+             (#sql_index{meta = #{ignore := true}}) ->
+                  [];
+             (I) ->
+                  [iolist_to_binary(
+                     [format_create_index(SchemaInfo, Table, I),
+                      <<"\n">>])]
           end,
           Table#sql_table.indices);
-format_create_table(mysql = DBType, DBVersion, Table) ->
+format_create_table(#sql_schema_info{db_type = mysql} = SchemaInfo, Table) ->
     TableName = Table#sql_table.name,
     [iolist_to_binary(
       [<<"CREATE TABLE ">>, TableName, <<" (\n">>,
        lists:join(
          <<",\n">>,
          lists:map(
-           fun(C) -> format_column_def(DBType, DBVersion, C) end,
-           Table#sql_table.columns)),
+           fun(C) -> format_column_def(SchemaInfo, C) end,
+           Table#sql_table.columns) ++
+             format_primary_key(SchemaInfo, Table)),
        <<"\n) ENGINE=InnoDB CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci;\n">>])] ++
-        lists:map(
-          fun(I) ->
-                  iolist_to_binary(
-                    [format_create_index(DBType, DBVersion, Table, I),
-                     <<"\n">>])
+        lists:flatmap(
+          fun(#sql_index{meta = #{primary_key := true}}) ->
+                  [];
+             (#sql_index{meta = #{ignore := true}}) ->
+                  [];
+             (I) ->
+                  [iolist_to_binary(
+                     [format_create_index(SchemaInfo, Table, I),
+                      <<"\n">>])]
           end,
           Table#sql_table.indices).
-%format_create_table(DBType, _DBVersion, Table) ->
-%    ?ERROR_MSG("Can't create SQL table ~p on ~p",
-%               [Table#sql_table.name, DBType]),
-%    error.
 
-create_table(Host, Table) ->
+create_table(Host, SchemaInfo, Table) ->
     ejabberd_sql:sql_query(
       Host,
-      fun(DBType, DBVersion) ->
-              SQLs = format_create_table(DBType, DBVersion, Table),
+      fun() ->
+              SQLs = format_create_table(SchemaInfo, Table),
               ?INFO_MSG("Creating table ~s:~n~s~n",
                         [Table#sql_table.name, SQLs]),
               lists:foreach(
@@ -636,20 +708,22 @@ create_table(Host, Table) ->
               case Table#sql_table.post_create of
                   undefined ->
                       ok;
-                  F ->
-                      F(DBType, DBVersion)
+                  F when is_function(F, 1) ->
+                      PostSQLs = F(SchemaInfo),
+                      lists:foreach(
+                        fun(SQL) -> ejabberd_sql:sql_query_t(SQL) end,
+                        PostSQLs)
               end
       end).
 
-create_tables(Host, Module, Schema) ->
+create_tables(Host, Module, SchemaInfo, Schema) ->
     lists:foreach(
       fun(Table) ->
-              Table2 = filter_table_sh(Table),
-              Res = create_table(Host, Table2),
+              Res = create_table(Host, SchemaInfo, Table),
               case Res of
                   {error, Error} ->
                       ?ERROR_MSG("Failed to create table ~s: ~p",
-                                 [Table2#sql_table.name, Error]),
+                                 [Table#sql_table.name, Error]),
                       error(Error);
                   _ ->
                       ok
@@ -677,9 +751,72 @@ should_update_schema(Host) ->
             false
     end.
 
-update_schema(Host, Module, Schemas) ->
+preprocess_table(SchemaInfo, Table) ->
+    Table1 = filter_table_sh(SchemaInfo, Table),
+    ImplicitPK =
+        case SchemaInfo#sql_schema_info.db_type of
+            pgsql -> false;
+            sqlite ->
+                case lists:keyfind(bigserial, #sql_column.type,
+                                   Table1#sql_table.columns) of
+                    false -> false;
+                    #sql_column{name = Name} -> {ok, Name}
+                end;
+            mysql ->
+                case lists:keyfind(bigserial, #sql_column.type,
+                                   Table1#sql_table.columns) of
+                    false -> false;
+                    #sql_column{name = Name} -> {ok, Name}
+                end
+        end,
+    Indices =
+        case ImplicitPK of
+            false ->
+                {Inds, _} =
+                    lists:mapfoldl(
+                      fun(#sql_index{unique = true} = I, false) ->
+                              {I#sql_index{
+                                 meta = (I#sql_index.meta)#{primary_key => true}},
+                               true};
+                         (I, Acc) ->
+                              {I, Acc}
+                      end, false, Table1#sql_table.indices),
+                Inds;
+            {ok, CN} ->
+                lists:map(
+                  fun(#sql_index{columns = [CN1]} = I) when CN == CN1 ->
+                          I#sql_index{
+                            meta = (I#sql_index.meta)#{ignore => true}};
+                     (I) -> I
+                  end,
+                  Table1#sql_table.indices)
+        end,
+    Table1#sql_table{indices = Indices}.
+
+preprocess_schemas(SchemaInfo, Schemas) ->
+    lists:map(
+      fun(Schema) ->
+              Schema#sql_schema{
+                tables = lists:map(
+                           fun(T) ->
+                                   preprocess_table(SchemaInfo, T)
+                           end,
+                           Schema#sql_schema.tables)}
+      end, Schemas).
+
+update_schema(Host, Module, RawSchemas) ->
     case should_update_schema(Host) of
         true ->
+            SchemaInfo =
+                ejabberd_sql:sql_query(
+                  Host,
+                  fun(DBType, DBVersion) ->
+                          #sql_schema_info{
+                             db_type = DBType,
+                             db_version = DBVersion,
+                             new_schema = ejabberd_sql:use_new_schema()}
+                  end),
+            Schemas = preprocess_schemas(SchemaInfo, RawSchemas),
             Version = get_current_version(Host, Module, Schemas),
             LastSchema = lists:max(Schemas),
             LastVersion = LastSchema#sql_schema.version,
@@ -687,7 +824,7 @@ update_schema(Host, Module, Schemas) ->
                 _ when Version < 0 ->
                     ?ERROR_MSG("Can't update SQL schema for module ~p, please do it manually", [Module]);
                 0 ->
-                    create_tables(Host, Module, LastSchema);
+                    create_tables(Host, Module, SchemaInfo, LastSchema);
                 LastVersion ->
                     ok;
                 _ when LastVersion < Version ->
@@ -697,7 +834,8 @@ update_schema(Host, Module, Schemas) ->
                       fun(Schema) ->
                               if
                                   Schema#sql_schema.version > Version ->
-                                      do_update_schema(Host, Module, Schema);
+                                      do_update_schema(Host, Module,
+                                                       SchemaInfo, Schema);
                                   true ->
                                       ok
                               end
@@ -707,7 +845,7 @@ update_schema(Host, Module, Schemas) ->
             ok
     end.
 
-do_update_schema(Host, Module, Schema) ->
+do_update_schema(Host, Module, SchemaInfo, Schema) ->
     lists:foreach(
       fun({add_column, TableName, ColumnName}) ->
               {value, Table} =
@@ -719,9 +857,9 @@ do_update_schema(Host, Module, Schema) ->
               Res =
                   ejabberd_sql:sql_query(
                     Host,
-                    fun(DBType, DBVersion) ->
-                            Def = format_column_def(DBType, DBVersion, Column),
-                            Default = format_default(DBType, DBVersion, Column),
+                    fun(DBType, _DBVersion) ->
+                            Def = format_column_def(SchemaInfo, Column),
+                            Default = format_default(SchemaInfo, Column),
                             SQLs =
                                 [[<<"ALTER TABLE ">>,
                                   TableName,
@@ -730,7 +868,7 @@ do_update_schema(Host, Module, Schema) ->
                                   <<" DEFAULT ">>,
                                   Default, <<";\n">>]] ++
                                 case Column#sql_column.default of
-                                    false ->
+                                    false when DBType /= sqlite ->
                                         [[<<"ALTER TABLE ">>,
                                           TableName,
                                           <<" ALTER COLUMN ">>,
@@ -779,45 +917,57 @@ do_update_schema(Host, Module, Schema) ->
                   _ ->
                       ok
               end;
-         ({create_index, TableName, Columns}) ->
-              {value, Table1} =
-                  lists:keysearch(
-                    TableName, #sql_table.name, Schema#sql_schema.tables),
-              {value, Index1} =
-                  lists:keysearch(
-                    Columns, #sql_index.columns, Table1#sql_table.indices),
-              Table = filter_table_sh(Table1),
-              Index =
+         ({create_index, TableName, Columns1}) ->
+              Columns =
                   case ejabberd_sql:use_new_schema() of
                       true ->
-                          Index1;
+                          Columns1;
                       false ->
-                          Index1#sql_index{
-                            columns =
-                                lists:delete(
-                                  <<"server_host">>, Index1#sql_index.columns)
-                           }
+                          lists:delete(
+                            <<"server_host">>, Columns1)
                   end,
-              Res =
-                  ejabberd_sql:sql_query(
-                    Host,
-                    fun(DBType, DBVersion) ->
-                            SQL1 = format_create_index(
-                                    DBType, DBVersion, Table, Index),
-                            SQL = iolist_to_binary(SQL1),
-                            ?INFO_MSG("Create index ~s/~p:~n~s~n",
-                                      [Table#sql_table.name,
-                                       Index#sql_index.columns,
-                                       SQL]),
-                            ejabberd_sql:sql_query_t(SQL)
-                    end),
-              case Res of
-                  {error, Error} ->
-                      ?ERROR_MSG("Failed to update table ~s: ~p",
-                                 [TableName, Error]),
-                      error(Error);
+              {value, Table} =
+                  lists:keysearch(
+                    TableName, #sql_table.name, Schema#sql_schema.tables),
+              {value, Index} =
+                  lists:keysearch(
+                    Columns, #sql_index.columns, Table#sql_table.indices),
+              case Index#sql_index.meta of
+                  #{ignore := true} -> ok;
                   _ ->
-                      ok
+                      Res =
+                          ejabberd_sql:sql_query(
+                            Host,
+                            fun() ->
+                                    case Index#sql_index.meta of
+                                        #{primary_key := true} ->
+                                            SQL1 = format_add_primary_key(
+                                                     SchemaInfo, Table, Index),
+                                            SQL = iolist_to_binary(SQL1),
+                                            ?INFO_MSG("Add primary key ~s/~p:~n~s~n",
+                                                      [Table#sql_table.name,
+                                                       Index#sql_index.columns,
+                                                       SQL]),
+                                            ejabberd_sql:sql_query_t(SQL);
+                                        _ ->
+                                            SQL1 = format_create_index(
+                                                     SchemaInfo, Table, Index),
+                                            SQL = iolist_to_binary(SQL1),
+                                            ?INFO_MSG("Create index ~s/~p:~n~s~n",
+                                                      [Table#sql_table.name,
+                                                       Index#sql_index.columns,
+                                                       SQL]),
+                                            ejabberd_sql:sql_query_t(SQL)
+                                    end
+                            end),
+                      case Res of
+                          {error, Error} ->
+                              ?ERROR_MSG("Failed to update table ~s: ~p",
+                                         [TableName, Error]),
+                              error(Error);
+                          _ ->
+                              ok
+                      end
               end;
          ({drop_index, TableName, Columns1}) ->
               Columns =
@@ -867,6 +1017,96 @@ do_update_schema(Host, Module, Schema) ->
       end, Schema#sql_schema.update),
     store_version(Host, Module, Schema#sql_schema.version).
 
+
+print_schema(SDBType, SDBVersion, SNewSchema) ->
+    {DBType, DBVersion} =
+        case SDBType of
+            "pgsql" ->
+                case string:split(SDBVersion, ".") of
+                    [SMajor, SMinor] ->
+                        try {list_to_integer(SMajor), list_to_integer(SMinor)} of
+                            {Major, Minor} ->
+                                {pgsql, Major * 10000 + Minor}
+                        catch _:_ ->
+                                io:format("pgsql version be in the form of "
+                                          "Major.Minor, e.g. 16.1~n"),
+                                {error, error}
+                        end;
+                    _ ->
+                        io:format("pgsql version be in the form of "
+                                  "Major.Minor, e.g. 16.1~n"),
+                        {error, error}
+                end;
+            "mysql" ->
+                case ejabberd_sql:parse_mysql_version(SDBVersion, 0) of
+                    {ok, V} ->
+                        {mysql, V};
+                    error ->
+                        io:format("mysql version be in the same form as "
+                                  "SELECT VERSION() returns, e.g. 8.2.0~n"),
+                        {error, error}
+                end;
+            "sqlite" ->
+                {sqlite, undefined};
+            _ ->
+                io:format("db_type must be one of the following: "
+                          "'pgsql', 'mysql', 'sqlite'~n"),
+                {error, error}
+        end,
+    NewSchema =
+        case SNewSchema of
+            "0" -> false;
+            "1" -> true;
+            "false" -> false;
+            "true" -> true;
+            _ ->
+                io:format("new_schema must be one of the following: "
+                          "'0', '1', 'false', 'true'~n"),
+                error
+        end,
+    case {DBType, NewSchema} of
+        {error, _} -> ?STATUS_ERROR;
+        {_, error} -> ?STATUS_ERROR;
+        _ ->
+            SchemaInfo =
+                #sql_schema_info{
+                   db_type = DBType,
+                   db_version = DBVersion,
+                   new_schema = NewSchema},
+            Mods = ejabberd_config:beams(all),
+            lists:foreach(
+              fun(Mod) ->
+                      case erlang:function_exported(Mod, sql_schemas, 0) of
+                          true ->
+                              Schemas = Mod:sql_schemas(),
+                              Schemas2 = preprocess_schemas(SchemaInfo, Schemas),
+                              Schema = lists:max(Schemas2),
+                              SQLs =
+                                  lists:flatmap(
+                                    fun(Table) ->
+                                            SQLs = format_create_table(SchemaInfo, Table),
+                                            PostSQLs =
+                                                case Table#sql_table.post_create of
+                                                    undefined ->
+                                                        [];
+                                                    F when is_function(F, 1) ->
+                                                        PSQLs = F(SchemaInfo),
+                                                        lists:map(
+                                                          fun(S) ->
+                                                                  [S, <<"\n">>]
+                                                          end, PSQLs)
+                                                end,
+                                            SQLs ++ PostSQLs
+                                    end, Schema#sql_schema.tables),
+                              io:format("~s~n", [SQLs]);
+                          false ->
+                              ok
+                      end
+              end, Mods),
+            ?STATUS_SUCCESS
+    end.
+
+
 test() ->
     Schemas =
         [#sql_schema{
@@ -888,7 +1128,9 @@ test() ->
                                #sql_column{name = <<"type">>, type = text},
                                #sql_column{name = <<"created_at">>, type = timestamp,
                                            default = true}],
-                    indices = [#sql_index{
+                    indices = [#sql_index{columns = [<<"id">>],
+                                          unique = true},
+                               #sql_index{
                                   columns = [<<"server_host">>, <<"username">>, <<"timestamp">>]},
                                #sql_index{
                                   columns = [<<"server_host">>, <<"username">>, <<"peer">>]},
@@ -905,7 +1147,8 @@ test() ->
                   [<<"server_host">>, <<"origin_id">>]},
                  {drop_index, <<"archive2">>,
                   [<<"server_host">>, <<"origin_id">>]},
-                 {drop_column, <<"archive2">>, <<"origin_id">>}
+                 {drop_column, <<"archive2">>, <<"origin_id">>},
+                 {create_index, <<"archive2">>, [<<"id">>]}
                 ]},
          #sql_schema{
             version = 1,
