@@ -5,7 +5,7 @@
 %%% Created : 19 Feb 2015 by Christophe Romain <christophe.romain@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2006-2023   ProcessOne
+%%% ejabberd, Copyright (C) 2006-2024   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -603,8 +603,8 @@ compile(LibDir) ->
     Er = [compile_erlang_file(Bin, File, Options)
           || File <- filelib:wildcard(Src++"/*.erl")],
     Ex = [compile_elixir_file(Bin, File)
-          || File <- filelib:wildcard(Lib ++ "/*.ex")],
-    compile_result(Er++Ex).
+          || File <- filelib:wildcard(Lib ++ "/**/*.ex")],
+    compile_result(lists:flatten([Er, Ex])).
 
 compile_c_files(LibDir) ->
     case file:read_file_info(filename:join(LibDir, "c_src/Makefile")) of
@@ -673,7 +673,7 @@ compile_elixir_file(Dest, File) when is_list(Dest) and is_list(File) ->
 
 compile_elixir_file(Dest, File) ->
   try 'Elixir.Kernel.ParallelCompiler':files_to_path([File], Dest, []) of
-    [Module] -> {ok, Module}
+    Modules when is_list(Modules) -> {ok, Modules}
   catch
     _ -> {error, {compilation_failed, File}}
   end.
@@ -823,7 +823,7 @@ write_commit_json(Url, RepDir) ->
     end.
 
 find_commit_json(Attrs) ->
-    {_, FromPath} = lists:keyfind(path, 1, Attrs),
+    FromPath = get_module_path(Attrs),
     case {find_commit_json_path(FromPath),
           find_commit_json_path(filename:join(FromPath, ".."))}
     of
@@ -946,7 +946,7 @@ get_page(Node, Query, Lang) ->
     Title ++ Result ++ Contents.
 
 get_module_home(Module, Attrs) ->
-    case element(2, lists:keyfind(home, 1, Attrs)) of
+    case get_module_information(home, Attrs) of
         "https://github.com/processone/ejabberd-contrib/tree/master/" = P1 ->
             P1 ++ atom_to_list(Module);
         Other ->
@@ -954,17 +954,26 @@ get_module_home(Module, Attrs) ->
     end.
 
 get_module_summary(Attrs) ->
-    element(2, lists:keyfind(summary, 1, Attrs)).
+    get_module_information(summary, Attrs).
 
 get_module_author(Attrs) ->
-    element(2, lists:keyfind(author, 1, Attrs)).
+    get_module_information(author, Attrs).
+
+get_module_path(Attrs) ->
+    get_module_information(path, Attrs).
+
+get_module_information(Attribute, Attrs) ->
+    case lists:keyfind(Attribute, 1, Attrs) of
+        false -> "";
+        {_, Value} -> Value
+    end.
 
 get_installed_module_el({ModAtom, Attrs}, Lang) ->
     Mod = misc:atom_to_binary(ModAtom),
     Home = list_to_binary(get_module_home(ModAtom, Attrs)),
     Summary = list_to_binary(get_module_summary(Attrs)),
     Author = list_to_binary(get_module_author(Attrs)),
-    {_, FromPath} = lists:keyfind(path, 1, Attrs),
+    FromPath = get_module_path(Attrs),
     FromFile = case find_commit_json_path(FromPath) of
                    {ok, FF} -> FF;
                    {error, _} -> "dummypath"
