@@ -5,7 +5,7 @@
 %%% Created :  9 Aug 2017 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -72,8 +72,7 @@ get_commands_spec() ->
                         args_example = [],
                         args_desc = [],
                         result = {res, rescode},
-                        result_example = ok,
-                        result_desc = "Status code: 0 on success, 1 otherwise"}
+                        result_example = ok}
     ].
 
 update_sql() ->
