@@ -5,7 +5,7 @@
 %%% Created : 19 Mar 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -1679,7 +1679,7 @@ mod_doc() ->
             #{value => ?T("Options"),
               note => "improved in 22.05",
               desc =>
-                  ?T("This option allows to define the desired "
+                  ?T("Define the "
                      "default room options. Note that the creator of a room "
                      "can modify the options of his room at any time using an "
                      "XMPP client with MUC capability. The 'Options' are:")},
@@ -1732,7 +1732,7 @@ mod_doc() ->
                     ?T("When a user tries to join a room where they have no "
                        "affiliation (not owner, admin or member), the room "
                        "requires them to fill a CAPTCHA challenge (see section "
-                       "https://docs.ejabberd.im/admin/configuration/#captcha[CAPTCHA] "
+                       "http://../#captcha[CAPTCHA] "
                        "in order to accept their join in the room. "
                        "The default value is 'false'.")}},
              {description,
