@@ -5,7 +5,7 @@
 %%% Created : 8 Sep 2007 by Badlop <badlop@ono.com>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -93,10 +93,11 @@ depends(_Host, _Opts) ->
 get_commands_spec() ->
     [
      #ejabberd_commands{name = muc_online_rooms, tags = [muc],
-		       desc = "List existing rooms ('global' to get all vhosts)",
+		       desc = "List existing rooms",
+		       longdesc = "Ask for a specific host, or `global` to use all vhosts.",
                        policy = admin,
 		       module = ?MODULE, function = muc_online_rooms,
-		       args_desc = ["MUC service, or 'global' for all"],
+		       args_desc = ["MUC service, or `global` for all"],
 		       args_example = ["muc.example.com"],
 		       result_desc = "List of rooms",
 		       result_example = ["room1@muc.example.com", "room2@muc.example.com"],
@@ -104,10 +105,11 @@ get_commands_spec() ->
 		       args_rename = [{host, service}],
 		       result = {rooms, {list, {room, string}}}},
 	#ejabberd_commands{name = muc_online_rooms_by_regex, tags = [muc],
-		       desc = "List existing rooms ('global' to get all vhosts) by regex",
+		       desc = "List existing rooms filtered by regexp",
+		       longdesc = "Ask for a specific host, or `global` to use all vhosts.",
                        policy = admin,
 		       module = ?MODULE, function = muc_online_rooms_by_regex,
-		       args_desc = ["MUC service, or 'global' for all",
+		       args_desc = ["MUC service, or `global` for all",
 			   				"Regex pattern for room name"],
 		       args_example = ["muc.example.com", "^prefix"],
 		       result_desc = "List of rooms with summary",
@@ -160,7 +162,7 @@ get_commands_spec() ->
 		       args_example = ["/home/ejabberd/rooms.txt"],
 		       args = [{file, string}],
 		       result = {res, rescode}},
-     #ejabberd_commands{name = create_room_with_opts, tags = [muc_room],
+     #ejabberd_commands{name = create_room_with_opts, tags = [muc_room, muc_sub],
 		       desc = "Create a MUC room name@service in host with given options",
 		       longdesc =
                         "The syntax of `affiliations` is: `Type:JID,Type:JID`. "
@@ -246,7 +248,7 @@ get_commands_spec() ->
 		        result_example = ["room1@muc.example.com", "room2@muc.example.com"],
 			args = [{user, binary}, {host, binary}],
 		        result = {rooms, {list, {room, string}}}},
-     #ejabberd_commands{name = get_user_subscriptions, tags = [muc],
+     #ejabberd_commands{name = get_user_subscriptions, tags = [muc, muc_sub],
 			desc = "Get the list of rooms where this user is subscribed",
 			note = "added in 21.04",
 			module = ?MODULE, function = get_user_subscriptions,
@@ -306,6 +308,23 @@ get_commands_spec() ->
 			args = [{name, binary}, {service, binary}, {password, binary},
 				{reason, binary}, {users, binary}],
 		        result = {res, rescode}},
+     #ejabberd_commands{name = send_direct_invitation, tags = [muc_room],
+			desc = "Send a direct invitation to several destinations",
+			longdesc = "Since ejabberd 20.12, this command is "
+                        "asynchronous: the API call may return before the "
+                        "server has send all the invitations.\n\n"
+                        "`password` and `message` can be set to `none`.",
+			module = ?MODULE, function = send_direct_invitation,
+			version = 1,
+			note = "updated in 24.02",
+		        args_desc = ["Room name", "MUC service", "Password, or `none`",
+			 "Reason text, or `none`", "List of users JIDs"],
+			args_example = [<<"room1">>, <<"muc.example.com">>,
+					<<>>, <<"Check this out!">>,
+					["user2@localhost", "user3@example.com"]],
+			args = [{name, binary}, {service, binary}, {password, binary},
+				{reason, binary}, {users, {list, {jid, binary}}}],
+		        result = {res, rescode}},
 
      #ejabberd_commands{name = change_room_option, tags = [muc_room],
 		       desc = "Change an option in a MUC room",
@@ -329,7 +348,7 @@ get_commands_spec() ->
 								 {value, string}
 								]}}
 						}}},
-     #ejabberd_commands{name = subscribe_room, tags = [muc_room],
+     #ejabberd_commands{name = subscribe_room, tags = [muc_room, muc_sub],
 			desc = "Subscribe to a MUC conference",
 			module = ?MODULE, function = subscribe_room,
 			args_desc = ["User JID", "a user's nick",
@@ -342,7 +361,22 @@ get_commands_spec() ->
 			args = [{user, binary}, {nick, binary}, {room, binary},
 				{nodes, binary}],
 			result = {nodes, {list, {node, string}}}},
-     #ejabberd_commands{name = subscribe_room_many, tags = [muc_room],
+     #ejabberd_commands{name = subscribe_room, tags = [muc_room, muc_sub],
+			desc = "Subscribe to a MUC conference",
+			module = ?MODULE, function = subscribe_room,
+			version = 1,
+			note = "updated in 24.02",
+			args_desc = ["User JID", "a user's nick",
+			    "the room to subscribe", "list of nodes"],
+			args_example = ["tom@localhost", "Tom", "room1@conference.localhost",
+			    ["urn:xmpp:mucsub:nodes:messages", "urn:xmpp:mucsub:nodes:affiliations"]],
+			result_desc = "The list of nodes that has subscribed",
+			result_example = ["urn:xmpp:mucsub:nodes:messages",
+			    "urn:xmpp:mucsub:nodes:affiliations"],
+			args = [{user, binary}, {nick, binary}, {room, binary},
+				{nodes, {list, {node, binary}}}],
+			result = {nodes, {list, {node, string}}}},
+     #ejabberd_commands{name = subscribe_room_many, tags = [muc_room, muc_sub],
 			desc = "Subscribe several users to a MUC conference",
 			note = "added in 22.05",
 			longdesc = "This command accepts up to 50 users at once "
@@ -365,14 +399,38 @@ get_commands_spec() ->
                                 {room, binary},
 				{nodes, binary}],
 			result = {res, rescode}},
-     #ejabberd_commands{name = unsubscribe_room, tags = [muc_room],
+     #ejabberd_commands{name = subscribe_room_many, tags = [muc_room, muc_sub],
+			desc = "Subscribe several users to a MUC conference",
+			longdesc = "This command accepts up to 50 users at once "
+                            "(this is configurable with the *`mod_muc_admin`* option "
+                            "`subscribe_room_many_max_users`)",
+			module = ?MODULE, function = subscribe_room_many,
+			version = 1,
+			note = "updated in 24.02",
+			args_desc = ["Users JIDs and nicks",
+                                     "the room to subscribe",
+                                     "nodes separated by commas: `,`"],
+			args_example = [[{"tom@localhost", "Tom"},
+                                         {"jerry@localhost", "Jerry"}],
+                                        "room1@conference.localhost",
+                                        ["urn:xmpp:mucsub:nodes:messages", "urn:xmpp:mucsub:nodes:affiliations"]],
+			args = [{users, {list,
+                                         {user, {tuple,
+                                                 [{jid, binary},
+                                                  {nick, binary}
+                                                 ]}}
+                                        }},
+                                {room, binary},
+				{nodes, {list, {node, binary}}}],
+			result = {res, rescode}},
+     #ejabberd_commands{name = unsubscribe_room, tags = [muc_room, muc_sub],
 			desc = "Unsubscribe from a MUC conference",
 			module = ?MODULE, function = unsubscribe_room,
 			args_desc = ["User JID", "the room to subscribe"],
 			args_example = ["tom@localhost", "room1@conference.localhost"],
 			args = [{user, binary}, {room, binary}],
 			result = {res, rescode}},
-     #ejabberd_commands{name = get_subscribers, tags = [muc_room],
+     #ejabberd_commands{name = get_subscribers, tags = [muc_room, muc_sub],
 			desc = "List subscribers of a MUC conference",
 			module = ?MODULE, function = get_subscribers,
 		        args_desc = ["Room name", "MUC service"],
@@ -1072,20 +1130,22 @@ get_room_occupants_number(Room, Host) ->
 %%----------------------------
 %% http://xmpp.org/extensions/xep-0249.html
 
-send_direct_invitation(RoomName, RoomService, Password, Reason, UsersString) ->
+send_direct_invitation(RoomName, RoomService, Password, Reason, UsersString) when is_binary(UsersString) ->
+    UsersStrings = binary:split(UsersString, <<":">>, [global]),
+    send_direct_invitation(RoomName, RoomService, Password, Reason, UsersStrings);
+send_direct_invitation(RoomName, RoomService, Password, Reason, UsersStrings) ->
     case jid:make(RoomName, RoomService) of
 	error ->
 	    throw({error, "Invalid 'roomname' or 'service'"});
 	RoomJid ->
 	    XmlEl = build_invitation(Password, Reason, RoomJid),
-	    Users = get_users_to_invite(RoomJid, UsersString),
+	    Users = get_users_to_invite(RoomJid, UsersStrings),
 	    [send_direct_invitation(RoomJid, UserJid, XmlEl)
 	     || UserJid <- Users],
 	    ok
     end.
 
-get_users_to_invite(RoomJid, UsersString) ->
-    UsersStrings = binary:split(UsersString, <<":">>, [global]),
+get_users_to_invite(RoomJid, UsersStrings) ->
     OccupantsTuples = get_room_occupants(RoomJid#jid.luser,
 					 RoomJid#jid.lserver),
     OccupantsJids = [jid:decode(JidString)
@@ -1437,8 +1497,10 @@ set_room_affiliation(Name, Service, JID, AffiliationString) ->
 
 subscribe_room(_User, Nick, _Room, _Nodes) when Nick == <<"">> ->
     throw({error, "Nickname must be set"});
-subscribe_room(User, Nick, Room, Nodes) ->
+subscribe_room(User, Nick, Room, Nodes) when is_binary(Nodes) ->
     NodeList = re:split(Nodes, "\\h*,\\h*"),
+    subscribe_room(User, Nick, Room, NodeList);
+subscribe_room(User, Nick, Room, NodeList) ->
     try jid:decode(Room) of
 	#jid{luser = Name, lserver = Host} when Name /= <<"">> ->
 	    try jid:decode(User) of
