@@ -5,7 +5,7 @@
 %%% Created :  5 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -61,7 +61,8 @@
 	 c2s_copy_session/2,
 	 webadmin_page/3,
 	 webadmin_user/4,
-	 webadmin_user_parse_query/5]).
+	 webadmin_user_parse_query/5,
+	 c2s_handle_bind2_inline/1]).
 
 -export([mod_opt_type/1, mod_options/1, mod_doc/0, depends/2]).
 
@@ -130,6 +131,7 @@ start(Host, Opts) ->
           {hook, disco_info, get_info, 50},
           {hook, c2s_handle_info, c2s_handle_info, 50},
           {hook, c2s_copy_session, c2s_copy_session, 50},
+	  {hook, c2s_handle_bind2_inline, c2s_handle_bind2_inline, 50},
           {hook, webadmin_page_host, webadmin_page, 50},
           {hook, webadmin_user, webadmin_user, 50},
           {hook, webadmin_user_parse_query,  webadmin_user_parse_query, 50},
@@ -297,6 +299,10 @@ c2s_copy_session(State, #{resend_offline := Flag}) ->
 c2s_copy_session(State, _) ->
     State.
 
+c2s_handle_bind2_inline({#{jid := #jid{luser = LUser, lserver = LServer}} = State, Els, Results}) ->
+    delete_all_msgs(LUser, LServer),
+    {State, Els, Results}.
+
 -spec handle_offline_query(iq()) -> iq().
 handle_offline_query(#iq{from = #jid{luser = U1, lserver = S1},
 			 to = #jid{luser = U2, lserver = S2},
