@@ -3,7 +3,7 @@
 %%% Created : 25 Dec 2016 by Evgeny Khramtsov <ekhramtsov@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -32,11 +32,16 @@
 -export([c2s_stream_started/2, c2s_stream_features/2,
 	 c2s_authenticated_packet/2, c2s_unauthenticated_packet/2,
 	 c2s_unbinded_packet/2, c2s_closed/2, c2s_terminated/2,
-	 c2s_handle_send/3, c2s_handle_info/2, c2s_handle_call/3,
-	 c2s_handle_recv/3]).
+	 c2s_handle_send/3, c2s_handle_info/2, c2s_handle_cast/2,
+	 c2s_handle_call/3, c2s_handle_recv/3, c2s_inline_features/2,
+	 c2s_handle_sasl2_inline/1, c2s_handle_sasl2_inline_post/3,
+	 c2s_handle_bind2_inline/1]).
 %% adjust pending session timeout / access queue
 -export([get_resume_timeout/1, set_resume_timeout/2, queue_find/2]).
 
+%% for sasl2 inline resume
+-export([has_resume_data/2, post_resume_tasks/1]).
+
 -include_lib("xmpp/include/xmpp.hrl").
 -include("logger.hrl").
 -include_lib("p1_utils/include/p1_queue.hrl").
@@ -65,13 +70,18 @@ start(_Host, Opts) ->
     init_cache(Opts),
     {ok, [{hook, c2s_stream_started, c2s_stream_started, 50},
           {hook, c2s_post_auth_features, c2s_stream_features, 50},
+	  {hook, c2s_inline_features, c2s_inline_features, 50},
           {hook, c2s_unauthenticated_packet, c2s_unauthenticated_packet, 50},
           {hook, c2s_unbinded_packet, c2s_unbinded_packet, 50},
           {hook, c2s_authenticated_packet, c2s_authenticated_packet, 50},
           {hook, c2s_handle_send, c2s_handle_send, 50},
           {hook, c2s_handle_recv, c2s_handle_recv, 50},
           {hook, c2s_handle_info, c2s_handle_info, 50},
+          {hook, c2s_handle_cast, c2s_handle_cast, 50},
           {hook, c2s_handle_call, c2s_handle_call, 50},
+	  {hook, c2s_handle_sasl2_inline, c2s_handle_sasl2_inline, 50},
+	  {hook, c2s_handle_sasl2_inline_post, c2s_handle_sasl2_inline_post, 50},
+	  {hook, c2s_handle_bind2_inline, c2s_handle_bind2_inline, 50},
           {hook, c2s_closed, c2s_closed, 50},
           {hook, c2s_terminated, c2s_terminated, 50}]}.
 
@@ -112,6 +122,46 @@ c2s_stream_features(Acc, Host) ->
 	    Acc
     end.
 
+c2s_inline_features({Sasl, Bind} = Acc, Host) ->
+    case gen_mod:is_loaded(Host, ?MODULE) of
+	true ->
+	    {[#feature_sm{xmlns = ?NS_STREAM_MGMT_3} | Sasl],
+	     [#bind2_feature{var = ?NS_STREAM_MGMT_3} | Bind]};
+	false ->
+	    Acc
+    end.
+
+c2s_handle_sasl2_inline({State, Els, Results} = Acc) ->
+    case lists:keytake(sm_resume, 1, Els) of
+	{value, Resume, Rest} ->
+	    case has_resume_data(State, Resume) of
+		{ok, NewState, Resumed} ->
+		    Rest2 = lists:keydelete(bind2_bind, 1, Rest),
+		    {NewState, Rest2, [Resumed | Results]};
+		{error, ResumeError, _Reason} ->
+		    {State, Els, [ResumeError | Results]}
+	    end;
+	_ ->
+	    Acc
+    end.
+
+c2s_handle_sasl2_inline_post(State, _Els, Results) ->
+    case lists:keyfind(sm_resumed, 1, Results) of
+	false ->
+	    State;
+	_ ->
+	    post_resume_tasks(State)
+    end.
+
+c2s_handle_bind2_inline({State, Els, Results}) ->
+    case lists:keyfind(sm_enable, 1, Els) of
+	#sm_enable{xmlns = XMLNS} = Pkt ->
+	    {State2, Res} = handle_enable_int(State#{mgmt_xmlns => XMLNS}, Pkt),
+	    {State2, Els, [Res | Results]};
+	_ ->
+	    {State, Els, Results}
+    end.
+
 c2s_unauthenticated_packet(#{lang := Lang} = State, Pkt) when ?is_sm_packet(Pkt) ->
     %% XEP-0198 says: "For client-to-server connections, the client MUST NOT
     %% attempt to enable stream management until after it has completed Resource
@@ -209,6 +259,13 @@ c2s_handle_send(#{mgmt_state := MgmtState, mod := Mod,
 c2s_handle_send(State, _Pkt, _Result) ->
     State.
 
+c2s_handle_cast(#{mgmt_state := active} = State, send_ping) ->
+    {stop, send_rack(State)};
+c2s_handle_cast(#{mgmt_state := pending} = State, send_ping) ->
+    {stop, State};
+c2s_handle_cast(State, _Msg) ->
+    State.
+
 c2s_handle_call(#{mgmt_id := MgmtID, mgmt_queue := Queue, mod := Mod} = State,
 		{resume_session, MgmtID}, From) ->
     State1 = State#{mgmt_queue => p1_queue:file_to_ram(Queue)},
@@ -352,28 +409,28 @@ perform_stream_mgmt(Pkt, #{mgmt_xmlns := Xmlns, lang := Lang} = State) ->
 				   xmlns = Xmlns})
     end.
 
--spec handle_enable(state(), sm_enable()) -> state().
-handle_enable(#{mgmt_timeout := DefaultTimeout,
-		mgmt_queue_type := QueueType,
-		mgmt_max_timeout := MaxTimeout,
-		mgmt_xmlns := Xmlns, jid := JID} = State,
-	      #sm_enable{resume = Resume, max = Max}) ->
+-spec handle_enable_int(state(), sm_enable()) -> {state(), sm_enabled()}.
+handle_enable_int(#{mgmt_timeout := DefaultTimeout,
+		    mgmt_queue_type := QueueType,
+		    mgmt_max_timeout := MaxTimeout,
+		    mgmt_xmlns := Xmlns, jid := JID} = State,
+		  #sm_enable{resume = Resume, max = Max}) ->
     State1 = State#{mgmt_id => make_id()},
     Timeout = if Resume == false ->
-		      0;
-		 Max /= undefined, Max > 0, Max*1000 =< MaxTimeout ->
+	0;
+		  Max /= undefined, Max > 0, Max*1000 =< MaxTimeout ->
 		      Max*1000;
-		 true ->
+		  true ->
 		      DefaultTimeout
 	      end,
     Res = if Timeout > 0 ->
-		  ?DEBUG("Stream management with resumption enabled for ~ts",
-			 [jid:encode(JID)]),
-		  #sm_enabled{xmlns = Xmlns,
-			      id = encode_id(State1),
-			      resume = true,
-			      max = Timeout div 1000};
-	     true ->
+	?DEBUG("Stream management with resumption enabled for ~ts",
+	       [jid:encode(JID)]),
+	#sm_enabled{xmlns = Xmlns,
+		    id = encode_id(State1),
+		    resume = true,
+		    max = Timeout div 1000};
+	      true ->
 		  ?DEBUG("Stream management without resumption enabled for ~ts",
 			 [jid:encode(JID)]),
 		  #sm_enabled{xmlns = Xmlns}
@@ -381,6 +438,11 @@ handle_enable(#{mgmt_timeout := DefaultTimeout,
     State2 = State1#{mgmt_state => active,
 		     mgmt_queue => p1_queue:new(QueueType),
 		     mgmt_timeout => Timeout},
+    {State2, Res}.
+
+-spec handle_enable(state(), sm_enable()) -> state().
+handle_enable(State, Enable) ->
+    {State2, Res} = handle_enable_int(State, Enable),
     send(State2, Res).
 
 -spec handle_r(state()) -> state().
@@ -394,39 +456,48 @@ handle_a(State, #sm_a{h = H}) ->
     resend_rack(State1).
 
 -spec handle_resume(state(), sm_resume()) -> {ok, state()} | {error, state()}.
-handle_resume(#{user := User, lserver := LServer,
-		lang := Lang, socket := Socket} = State,
-	      #sm_resume{h = H, previd = PrevID, xmlns = Xmlns}) ->
-    R = case inherit_session_state(State, PrevID) of
-	    {ok, InheritedState} ->
-		{ok, InheritedState, H};
-	    {error, Err, InH} ->
-		{error, #sm_failed{reason = 'item-not-found',
-				   text = xmpp:mk_text(format_error(Err), Lang),
-				   h = InH, xmlns = Xmlns}, Err};
-	    {error, Err} ->
-		{error, #sm_failed{reason = 'item-not-found',
-				   text = xmpp:mk_text(format_error(Err), Lang),
-				   xmlns = Xmlns}, Err}
-	end,
-    case R of
-	{ok, #{jid := JID} = ResumedState, NumHandled} ->
-	    State1 = check_h_attribute(ResumedState, NumHandled),
-	    #{mgmt_xmlns := AttrXmlns, mgmt_stanzas_in := AttrH} = State1,
-	    State2 = send(State1, #sm_resumed{xmlns = AttrXmlns,
-					      h = AttrH,
-					      previd = PrevID}),
-	    State3 = resend_unacked_stanzas(State2),
-	    State4 = send(State3, #sm_r{xmlns = AttrXmlns}),
-	    State5 = ejabberd_hooks:run_fold(c2s_session_resumed, LServer, State4, []),
-	    ?INFO_MSG("(~ts) Resumed session for ~ts",
-		      [xmpp_socket:pp(Socket), jid:encode(JID)]),
-	    {ok, State5};
+handle_resume(#{user := User, lserver := LServer} = State,
+	      #sm_resume{} = Resume) ->
+    case has_resume_data(State, Resume) of
+	{ok, ResumedState, ResumedEl} ->
+	    State2 = send(ResumedState, ResumedEl),
+	    {ok, post_resume_tasks(State2)};
 	{error, El, Reason} ->
 	    log_resumption_error(User, LServer, Reason),
 	    {error, send(State, El)}
     end.
 
+-spec has_resume_data(state(), sm_resume()) ->
+    {ok, state(), sm_resumed()} | {error, sm_failed(), error_reason()}.
+has_resume_data(#{lang := Lang} = State,
+		#sm_resume{h = H, previd = PrevID, xmlns = Xmlns}) ->
+    case inherit_session_state(State, PrevID) of
+	{ok, InheritedState} ->
+	    State1 = check_h_attribute(InheritedState, H),
+	    #{mgmt_xmlns := AttrXmlns, mgmt_stanzas_in := AttrH} = State1,
+	    {ok, State1, #sm_resumed{xmlns = AttrXmlns,
+				     h = AttrH,
+				     previd = PrevID}};
+	{error, Err, InH} ->
+	    {error, #sm_failed{reason = 'item-not-found',
+			       text = xmpp:mk_text(format_error(Err), Lang),
+			       h = InH, xmlns = Xmlns}, Err};
+	{error, Err} ->
+	    {error, #sm_failed{reason = 'item-not-found',
+			       text = xmpp:mk_text(format_error(Err), Lang),
+			       xmlns = Xmlns}, Err}
+    end.
+
+-spec post_resume_tasks(state()) -> state().
+post_resume_tasks(#{lserver := LServer, socket := Socket, jid := JID,
+		    mgmt_xmlns := AttrXmlns} = State) ->
+    State3 = resend_unacked_stanzas(State),
+    State4 = send(State3, #sm_r{xmlns = AttrXmlns}),
+    State5 = ejabberd_hooks:run_fold(c2s_session_resumed, LServer, State4, []),
+    ?INFO_MSG("(~ts) Resumed session for ~ts",
+	      [xmpp_socket:pp(Socket), jid:encode(JID)]),
+    State5.
+
 -spec transition_to_pending(state(), _) -> state().
 transition_to_pending(#{mgmt_state := active, mod := Mod,
 			mgmt_timeout := 0} = State, _Reason) ->
